Package: PhIPData
Type: Package
Title: Container for PhIP-Seq Experiments
Version: 1.21.0
Authors@R: c(person("Athena", "Chen", email = "achen70@jhu.edu", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6900-2264")),
    person("Rob", "Scharpf", email = "rscharpf@jhu.edu", 
    role = c("aut")), 
    person("Ingo", "Ruczinski", email = "ingo@jhu.edu", 
    role = c("aut")))
Description: PhIPData defines an S4 class for phage-immunoprecipitation 
    sequencing (PhIP-seq) experiments. Buliding upon the 
    RangedSummarizedExperiment class, PhIPData enables users to coordinate 
    metadata with experimental data in analyses. Additionally, PhIPData 
    provides specialized methods to subset and identify beads-only samples, 
    subset objects using virus aliases, and use existing peptide libraries to 
    populate object parameters. 
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.1.0), SummarizedExperiment (>= 1.3.81)
Imports: BiocFileCache, BiocGenerics, methods, GenomicRanges, IRanges,
        S4Vectors, edgeR, cli, utils
Suggests: BiocStyle, testthat, knitr, rmarkdown, covr, dplyr, readr,
        withr
biocViews: Infrastructure, DataRepresentation, Sequencing, Coverage
BugReports: https://github.com/athchen/PhIPData/issues
Collate: 'defineBeads.R' 'PhIPData-class.R' 'alias.R' 'library.R'
        'subset.R' 'summaries.R' 'zzz.R'
RoxygenNote: 7.1.2
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/PhIPData
git_branch: devel
git_last_commit: 6c7acec
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:20:19 UTC; biocbuild
Author: Athena Chen [aut, cre] (ORCID: <https://orcid.org/0000-0001-6900-2264>),
  Rob Scharpf [aut],
  Ingo Ruczinski [aut]
Maintainer: Athena Chen <achen70@jhu.edu>
