# Generated by roxygen2: do not edit by hand

export("logfc<-")
export("peptideInfo<-")
export("prob<-")
export("sampleInfo<-")
export(PhIPData)
export(deleteAlias)
export(getAlias)
export(getBeadsName)
export(getLibrary)
export(librarySize)
export(listLibrary)
export(logfc)
export(makeLibrary)
export(peptideInfo)
export(prob)
export(propReads)
export(removeLibrary)
export(sampleInfo)
export(setAlias)
export(setBeadsName)
export(subsetBeads)
exportMethods("assay<-")
exportMethods("assays<-")
exportMethods("counts<-")
exportMethods("logfc<-")
exportMethods("peptideInfo<-")
exportMethods("prob<-")
exportMethods("sampleInfo<-")
exportMethods(counts)
exportMethods(logfc)
exportMethods(peptideInfo)
exportMethods(prob)
exportMethods(sampleInfo)
import(S4Vectors)
import(SummarizedExperiment)
import(methods)
importClassesFrom(edgeR,DGEList)
importFrom(BiocGenerics,"counts<-")
importFrom(BiocGenerics,counts)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,isEmpty)
importFrom(S4Vectors,setValidity2)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_warning)
